#include <iostream>
#include <fstream>
#include <vector>
#include <string>
#include <cstdlib>
#include <cmath>
#include <sstream>

#include "LHAPDF/LHAPDF.h"

#include "TCanvas.h"
#include "TGraphErrors.h"
#include "TGraphAsymmErrors.h"
#include "TAxis.h"
#include "TH1F.h"
#include "TROOT.h"
#include "TStyle.h"
#include "TString.h"
#include "TLegend.h"
#include "TPaveStats.h"
#include "TMultiGraph.h"

using namespace std;

int main()
{  
  cout << "\n ****************************************\n";
  cout <<   " * Computation of PDFs *\n";
  cout <<   " ****************************************\n\n";
  cout << endl;

  /////////////////////////
  //   General Options   //
   
 
  //////////////////////////
  // Important variables  //
  //////////////////////////
  double const xmin = 0.2;
  double const xmax = 0.99;
  int const nx = 100;       

  cout << "\n Settings of PDF computation" << endl;
  cout << " xmin = " << xmin << endl;
  cout << " xmax = " << xmax << endl;
  cout << endl;

  double const pdfmin=1e-30;
 
  LHAPDF::setVerbosity(0);
  
  for(int iplot=0;iplot<1;iplot++){
    
    double Q; // Scale where PDFs are compared
    string runname;
    string title;
    
    // Init PDF for CV and Error
    vector<string> pdfcvname;
    vector<string> pdftype;
    vector<string> legendname;

    if(iplot==0){

      pdfcvname.push_back("NNPDF31_nnlo_as_0118");
      legendname.push_back("NNPDF3.1");
      pdftype.push_back("ER_MC");
      
      pdfcvname.push_back("MMHT2014nnlo68cl");
      legendname.push_back("MMHT14");
      pdftype.push_back("ER_MSTW");

      pdfcvname.push_back("CT14nnlo");
      legendname.push_back("CT14");
      pdftype.push_back("ER_CTEQ");
            
      pdfcvname.push_back("ABMP16als118_5_nnlo");
      legendname.push_back("ABMP16");
      pdftype.push_back("ER_SYMMHESSIAN");
      
      runname="pdfcomp-highQ";
      
      title="NNLO, #alpha_{S}=0.118, Q = 100 GeV";
      
      Q = 100;//pow(1e4,0.5);
      
    }

    if(iplot==1){

      pdfcvname.push_back("NNPDF31_nnlo_as_0118");
      legendname.push_back("NNPDF3.1");
      pdftype.push_back("ER_MC68CL");
      
      pdfcvname.push_back("MMHT2014nnlo68cl");
      legendname.push_back("MMHT14");
      pdftype.push_back("ER_MSTW");
      
      pdfcvname.push_back("CT14nnlo");
      legendname.push_back("CT14");
      pdftype.push_back("ER_CTEQ");
      
      runname="pdfcomp-lowQ";
      
      title="NNLO, #alpha_{S}=0.118, Q = 1.7 GeV";
      
      Q = 1.7;//pow(1e4,0.5);
      
    }

    
      

    cout<<"PDF set used = "<<endl;
  for(unsigned i=0;i< pdfcvname.size();i++){
    cout<< pdfcvname.at(i)<<endl;
  }

  vector<Color_t> color;
  color.push_back(kGreen);
  color.push_back(kRed);
  color.push_back(kBlue);
  color.push_back(kViolet);

  vector<Color_t> color2;
  color2.push_back(kGreen+2);
  color2.push_back(kRed+1);
  color2.push_back(kBlue+1);
  color2.push_back(kViolet+1);

  vector<string> pdfname;
  pdfname.push_back("xg");
  pdfname.push_back("xu");
  pdfname.push_back("xd");
  pdfname.push_back("xsinglet");
  pdfname.push_back("xtriplet");
  pdfname.push_back("xvalence");
  pdfname.push_back("xsp");
  pdfname.push_back("xubar");
  pdfname.push_back("xdbar");
  pdfname.push_back("xc");
  pdfname.push_back("xDeltaS");
  pdfname.push_back("xsm");
  pdfname.push_back("xs");
  pdfname.push_back("xsbar");
  
  vector<string> outfilename;
  vector<string> outfilename2;
  vector<string> outfilenameE;
  vector<string> outfilename2E;
  vector<string> outfilename_abs;
  vector<string> outfilename2_abs;
  for(unsigned i=0;i< pdfname.size();i++){
    outfilename.push_back(pdfname.at(i)+"-"+runname+".eps");
    outfilename2.push_back(pdfname.at(i)+"-"+runname+".pdf");
    outfilenameE.push_back(pdfname.at(i)+"-ERR-"+runname+".eps");
    outfilename2E.push_back(pdfname.at(i)+"-ERR-"+runname+".pdf");
    outfilename_abs.push_back(pdfname.at(i)+"-abs-"+runname+".eps");
    outfilename2_abs.push_back(pdfname.at(i)+"-abs-"+runname+".pdf");
  }
  
  int mxabs=2;
  int iabs_init=0;
  //if(Q > 3) mxabs=1; // Only ratio plots for high scales
  //if(Q < 3) iabs_init=1; // Only ratio plots for high scales
  
  // Flavor map
  int const flavmap[] = {0,2,1};

  vector<string> titleY_abs;
  titleY_abs.push_back("x g ( x, Q^{2})");
  titleY_abs.push_back("x u ( x, Q^{2})");
  titleY_abs.push_back("x d ( x, Q^{2})");
  titleY_abs.push_back("x #Sigma ( x, Q^{2})");
  titleY_abs.push_back("x T_{3} ( x, Q^{2})");
  titleY_abs.push_back("x V ( x, Q^{2})");
  titleY_abs.push_back("x s^{+} ( x, Q^{2})");
  titleY_abs.push_back("x #bar{u} ( x, Q^{2})");
  titleY_abs.push_back("x #bar{d} ( x, Q^{2})");
  titleY_abs.push_back("x c^{+} ( x, Q^{2})");
  titleY_abs.push_back("x #Delta_{S} ( x, Q^{2})");
  titleY_abs.push_back("x s^{-} ( x, Q^{2})");
  titleY_abs.push_back("x s ( x, Q^{2})");
  titleY_abs.push_back("x #bar{s} ( x, Q^{2})");

  vector<string> titleY;
  titleY.push_back("g ( x, Q^{2}) / g ( x, Q^{2}) [ref] ");
  titleY.push_back("u ( x, Q^{2}) / u ( x, Q^{2}) [ref] ");
  titleY.push_back("d ( x, Q^{2}) / d ( x, Q^{2}) [ref] ");
  titleY.push_back("#Sigma ( x, Q^{2}) / #Sigma ( x, Q^{2}) [ref] ");
  titleY.push_back("T_{3} ( x, Q^{2}) / T_{3} ( x, Q^{2}) [ref] ");
  titleY.push_back("V ( x, Q^{2}) / V ( x, Q^{2}) [ref] ");
  titleY.push_back("s^{+} ( x, Q^{2}) / s^{+} ( x, Q^{2}) [ref] ");
  titleY.push_back("#bar{u} ( x, Q^{2}) / #bar{u} ( x, Q^{2}) [ref] ");
  titleY.push_back("#bar{d} ( x, Q^{2}) / #bar{d} ( x, Q^{2}) [ref] ");
  titleY.push_back("c^{+} ( x, Q^{2}) / c^{+} ( x, Q^{2}) [ref] ");
  titleY.push_back("#Delta_{S} ( x, Q^{2}) / #Delta_{S} ( x, Q^{2}) [ref] ");
  titleY.push_back("s^{-} ( x, Q^{2}) / s^{-} ( x, Q^{2}) [ref] ");
  titleY.push_back("s ( x, Q^{2}) / s ( x, Q^{2}) [ref] ");
  titleY.push_back("#bar{s} ( x, Q^{2}) / #bar{s} ( x, Q^{2}) [ref] ");
  
  vector<string> titleYE;
  titleYE.push_back("#delta g ( x, Q^{2}) / ( g ( x, Q^{2}) [ref] ) ");
  titleYE.push_back("#delta u ( x, Q^{2}) / ( u ( x, Q^{2}) [ref] ) ");
  titleYE.push_back("#delta d ( x, Q^{2}) / ( d ( x, Q^{2}) [ref] ) ");
  titleYE.push_back("#delta #Sigma ( x, Q^{2}) / ( #Sigma ( x, Q^{2}) [ref] )");
  titleYE.push_back("#delta T_{3} ( x, Q^{2}) / ( T_{3} ( x, Q^{2}) [ref] )");
  titleYE.push_back("#delta V ( x, Q^{2}) / ( V ( x, Q^{2}) [ref] )");
  titleYE.push_back("#delta s^{+} ( x, Q^{2}) / ( s^{+} ( x, Q^{2}) [ref] )");
  titleYE.push_back("#delta #bar{u} ( x, Q^{2}) / ( #bar{u} ( x, Q^{2}) [ref] )");
  titleYE.push_back("#delta #bar{d} ( x, Q^{2}) / ( #bar{d} ( x, Q^{2}) [ref] )");
  titleYE.push_back("#delta c^{+} ( x, Q^{2}) / ( c^{+} ( x, Q^{2}) [ref] )");
  titleYE.push_back("#delta #Delta_{S} ( x, Q^{2}) / ( #Delta_{S} ( x, Q^{2}) [ref] )");
  titleYE.push_back("#delta s^{-} ( x, Q^{2}) / ( s^{-} ( x, Q^{2}) [ref] )");
  titleYE.push_back("#delta s ( x, Q^{2}) / ( s ( x, Q^{2}) [ref] )");
  titleYE.push_back("#delta #bar{s} ( x, Q^{2}) / ( #bar{s} ( x, Q^{2}) [ref] )");
  
  cout << "\n Computing PDFs" << endl;
  
  cout << endl;

  const int histoFillStyle[] = { 1001, 3005, 3004 , 3003 };
  
  for (size_t l = 0; l < pdfname.size(); l++)
    //for (size_t l = 0; l < 1; l++)   
     {

       if( l!= 1 ) continue;
       
      cout << "COMPUTING AND PLOTTING " << pdfname[l] << " PDFs..." << endl;
      
      vector<TCanvas*> c;
      vector<TLegend*> leg;
      vector<TLegend*> leg_abs;

      // Ratio plots
      vector<TGraphErrors*> cvLux;
      vector<TGraphErrors*> cvLuxUp;
      vector<TGraphErrors*> cvLuxDn;
      vector<TGraphErrors*> cvLuxCT;

      // Absolute PDF plots
      vector<TGraphErrors*> cvLux_abs;
      vector<TGraphErrors*> cvLuxUp_abs;
      vector<TGraphErrors*> cvLuxDn_abs;
      vector<TGraphErrors*> cvLuxCT_abs;

      vector<TCanvas*> cE;
      vector<TLegend*> legE;
      vector<TGraph*> cvLuxE;
    
      double *CVpdf = new double[nx];
      double *ERRpdf = new double[nx];

      // Initialization
      for (int w = 0; w < nx; w++) {
	CVpdf[w] = 1.0;
	ERRpdf[w] = 1.0;
      }

      for(unsigned iabs=iabs_init;iabs<mxabs;iabs++){

	std::cout<<"iabs = "<<iabs<<std::endl;
      
	for (size_t iPdf = 0; iPdf < pdfcvname.size(); iPdf++)
        {
	  
          double *x = new double[nx];

          cvLux.push_back(new TGraphErrors(nx));
          cvLuxUp.push_back(new TGraphErrors(nx));
          cvLuxDn.push_back(new TGraphErrors(nx));
          cvLuxCT.push_back(new TGraphErrors(nx));

	  cvLux_abs.push_back(new TGraphErrors(nx));
          cvLuxUp_abs.push_back(new TGraphErrors(nx));
          cvLuxDn_abs.push_back(new TGraphErrors(nx));
          cvLuxCT_abs.push_back(new TGraphErrors(nx));

	  cvLuxE.push_back(new TGraph(nx));

          double cv[nx]={0.0};
	  double err[nx]={0.0};
	  
	  // For NNPDF errors - one sigma
          if (pdftype[iPdf] == "ER_MC")
            {
	      
	      LHAPDF::initPDFSet(pdfcvname[iPdf]);
	      int repfinal = LHAPDF::numberPDF();

	      cout<<"\n *************** \n"<<endl;
	      cout<<"Nrep = "<<repfinal<<endl;
	      cout<<"\n *************** \n"<<endl;
	      	      
	      double xpdflh_ref[nx][101]={{0.0}};
	      
	      for (int nrep = 0; nrep < repfinal; nrep++)
		{
		  LHAPDF::initPDF(nrep+1);
		  
		  for (int ix = 0; ix < nx; ix++)
		    {

		      double xpdflh=0;
		      double dexp=double(ix)/(nx-1);
		      x[ix] = xmin*pow(xmax/xmin,dexp);
		     		
		      if(l<3){
			int ifl=flavmap[l];
			xpdflh=LHAPDF::xfx(x[ix],Q,ifl);
		      }
		      if(l==3){ // Singlet
			xpdflh =0;
			for(unsigned i=1;i<7;i++){
			  xpdflh=xpdflh + LHAPDF::xfx(x[ix],Q,i) + LHAPDF::xfx(x[ix],Q,-i);
			}
		      }
		      if(l==4){ // Triplet
			xpdflh=( LHAPDF::xfx(x[ix],Q,2) + LHAPDF::xfx(x[ix],Q,-2)
				 -  LHAPDF::xfx(x[ix],Q,1) - LHAPDF::xfx(x[ix],Q,-1) );
		      }
		      if(l==5){ // Valence
			xpdflh =0;
			for(unsigned i=1;i<7;i++){
			  xpdflh=xpdflh + LHAPDF::xfx(x[ix],Q,i) - LHAPDF::xfx(x[ix],Q,-i);
			}
		      }
		      if(l==6){ // Triplet
			xpdflh=LHAPDF::xfx(x[ix],Q,3) + LHAPDF::xfx(x[ix],Q,-3);
		      }
		      if(l==7){ // Triplet
			xpdflh=LHAPDF::xfx(x[ix],Q,-2);
		      }
		      if(l==8){ // Triplet
			xpdflh=LHAPDF::xfx(x[ix],Q,-1);
		      }
		      if(l==9){ // Triplet
			xpdflh=LHAPDF::xfx(x[ix],Q,4)+LHAPDF::xfx(x[ix],Q,-4);
		      }
		       if(l==10){ 
			xpdflh=LHAPDF::xfx(x[ix],Q,-1)-LHAPDF::xfx(x[ix],Q,-2);
		      }
		       if(l==11){ 
			xpdflh=LHAPDF::xfx(x[ix],Q,3)-LHAPDF::xfx(x[ix],Q,-3);
		      }
		       if(l==12){ 
			 xpdflh=LHAPDF::xfx(x[ix],Q,3);
		       }
		       if(l==13){ 
			 xpdflh=LHAPDF::xfx(x[ix],Q,-3);
		       }
		      xpdflh_ref[ix][nrep] = xpdflh;

		      
		    } //end loop over ix
		  
		} // end loop over replicas
	      
	      // Now loop again over x
	      for(int ix=0; ix<nx; ix++)
		{

		  cv[ix]=0.0;
		  err[ix]=0.0;
		  for (int nrep = 0; nrep < repfinal; nrep++)
		    {
		      cv[ix] += xpdflh_ref[ix][nrep]/repfinal;
		      err[ix] += pow(xpdflh_ref[ix][nrep],2.0)/repfinal;
		    }
		  
		  err[ix] = pow(err[ix]-pow(cv[ix],2.0),0.5);

		  if (iPdf == 0) {
		    CVpdf[ix] = cv[ix];
		    ERRpdf[ix] = err[ix];
		  }
		  if( CVpdf[ix] < pdfmin ) CVpdf[ix]=pdfmin;
		   				  		  
		  cvLux[iPdf]->SetPoint(ix, x[ix], cv[ix]/CVpdf[ix]);
		  cvLux[iPdf]->SetPointError(ix, 0.0, err[ix]/CVpdf[ix]);
		  cvLuxCT[iPdf]->SetPoint(ix, x[ix], cv[ix]/CVpdf[ix]);
		  cvLuxUp[iPdf]->SetPoint(ix, x[ix], (cv[ix]+err[ix])/CVpdf[ix]);
		  cvLuxDn[iPdf]->SetPoint(ix, x[ix], (cv[ix]-err[ix])/CVpdf[ix]);
		  
		  cvLux_abs[iPdf]->SetPoint(ix, x[ix], cv[ix]);
		  cvLux_abs[iPdf]->SetPointError(ix, 0.0, err[ix]);
		  cvLuxCT_abs[iPdf]->SetPoint(ix, x[ix], cv[ix]);
		  cvLuxUp_abs[iPdf]->SetPoint(ix, x[ix], (cv[ix]+err[ix]));
		  cvLuxDn_abs[iPdf]->SetPoint(ix, x[ix], (cv[ix]-err[ix]));

		  // Relative error: PDF error normalized to the corresponding central value
		  //cvLuxE[iPdf]->SetPoint(ix, x[ix], err[ix]/cv[ix]);
		  cvLuxE[iPdf]->SetPoint(ix, x[ix], err[ix]/CVpdf[ix]);
		  //cvLuxE[iPdf]->SetPoint(ix, x[ix], err[ix]/ERRpdf[ix]);
		  //cvLuxE[iPdf]->SetPoint(ix, x[ix], err[ix]);

			} // end loop over ix
            } // end if pdftype
	  
	  // For NNPDF errors - 68% CL
          else if (pdftype[iPdf] == "ER_MC68CL")
            {
	      
	      LHAPDF::initPDFSet(pdfcvname[iPdf]);
	      int repfinal = LHAPDF::numberPDF();
	      
	      double xpdflh_ref[nx][101]={{0.0}};
	      
	      for (int nrep = 0; nrep < repfinal; nrep++)
		{
		  LHAPDF::initPDF(nrep+1);
		  
		  for (int ix = 0; ix < nx; ix++)
		    {

		      double xpdflh=0;
		      double dexp=double(ix)/(nx-1);
		      x[ix] = xmin*pow(xmax/xmin,dexp);
		      
		       if(l<3){
			int ifl=flavmap[l];
			xpdflh=LHAPDF::xfx(x[ix],Q,ifl);
		      }
		      if(l==3){ // Singlet
			xpdflh =0;
			for(unsigned i=1;i<7;i++){
			  xpdflh=xpdflh + LHAPDF::xfx(x[ix],Q,i) + LHAPDF::xfx(x[ix],Q,-i);
			}
		      }
		      if(l==4){ // Triplet
			xpdflh=( LHAPDF::xfx(x[ix],Q,2) + LHAPDF::xfx(x[ix],Q,-2)
				 -  LHAPDF::xfx(x[ix],Q,1) - LHAPDF::xfx(x[ix],Q,-1) );
		      }
		      if(l==5){ // Valence
			xpdflh =0;
			for(unsigned i=1;i<7;i++){
			  xpdflh=xpdflh + LHAPDF::xfx(x[ix],Q,i) - LHAPDF::xfx(x[ix],Q,-i);
			}
		      }
		      if(l==6){ // Triplet
			xpdflh=LHAPDF::xfx(x[ix],Q,3) + LHAPDF::xfx(x[ix],Q,-3);
		      }
		      if(l==7){ // Triplet
			xpdflh=LHAPDF::xfx(x[ix],Q,-2);
		      }
		      if(l==8){ // Triplet
			xpdflh=LHAPDF::xfx(x[ix],Q,-1);
		      }
		      if(l==9){ // Triplet
			xpdflh=LHAPDF::xfx(x[ix],Q,4)+LHAPDF::xfx(x[ix],Q,-4);
		      }
		       if(l==10){ 
			xpdflh=LHAPDF::xfx(x[ix],Q,-1)-LHAPDF::xfx(x[ix],Q,-2);
		      }
		        if(l==11){ 
			xpdflh=LHAPDF::xfx(x[ix],Q,3)-LHAPDF::xfx(x[ix],Q,-3);
		      }
			 if(l==12){ 
			 xpdflh=LHAPDF::xfx(x[ix],Q,3);
		       }
		       if(l==13){ 
			 xpdflh=LHAPDF::xfx(x[ix],Q,-3);
		       }
		      xpdflh_ref[ix][nrep] = xpdflh;

		    } //End loop over x

		} //End loop over rep

	      // Now for the plot
	      for (int ix = 0; ix < nx; ix++)
                {

		  vector<double> pdf_values;
		  for (int nrep = 0; nrep < repfinal; nrep++)
		    {
		      
		      // Put all numbers in a vector
		      pdf_values.push_back(xpdflh_ref[ix][nrep]);
		      
		    }
		  // Now sort the vector
		  sort(pdf_values.begin(),pdf_values.end());
		  
		  // Compute the central value as the midpoint of the 68% CL interval
		  int rep_min=repfinal*0.16;
		  int rep_max=repfinal*0.84;
		  cv[ix]= 0.5*(pdf_values.at(rep_min)+pdf_values.at(rep_max));
		  err[ix]= 0.5*(pdf_values.at(rep_max)-pdf_values.at(rep_min));
		  
		  if (iPdf == 0) CVpdf[ix] = cv[ix];
		  if( CVpdf[ix] < pdfmin ) CVpdf[ix]=pdfmin;
		  
                  cvLux[iPdf]->SetPoint(ix, x[ix], cv[ix]/CVpdf[ix]);
                  cvLux[iPdf]->SetPointError(ix, 0.0, err[ix]/CVpdf[ix]);
                  cvLuxCT[iPdf]->SetPoint(ix, x[ix], cv[ix]/CVpdf[ix]);
                  cvLuxUp[iPdf]->SetPoint(ix, x[ix], (cv[ix]+err[ix])/CVpdf[ix]);
                  cvLuxDn[iPdf]->SetPoint(ix, x[ix], (cv[ix]-err[ix])/CVpdf[ix]);

		  cvLux_abs[iPdf]->SetPoint(ix, x[ix], cv[ix]);
                  cvLux_abs[iPdf]->SetPointError(ix, 0.0, err[ix]);
                  cvLuxCT_abs[iPdf]->SetPoint(ix, x[ix], cv[ix]);
                  cvLuxUp_abs[iPdf]->SetPoint(ix, x[ix], (cv[ix]+err[ix]));
                  cvLuxDn_abs[iPdf]->SetPoint(ix, x[ix], (cv[ix]-err[ix]));

		  // Relative error: PDF error normalized to the corresponding central value
		  cvLuxE[iPdf]->SetPoint(ix, x[ix], err[ix]/cv[ix]);
		  //cvLuxE[iPdf]->SetPoint(ix, x[ix], err[ix]);
		  
		  
		}
            }

	  else if (pdftype[iPdf] == "ER_SYMMHESSIAN")
            {

	      LHAPDF::initPDFSet(pdfcvname[iPdf]);
	      int repfinal = LHAPDF::numberPDF();

	      double xpdflh_ref[nx][1001]={{0.0}};
	      
	      for (int nrep = 0; nrep <= repfinal; nrep++)
		{
		  LHAPDF::initPDF(nrep);
		  
		  for (int ix = 0; ix < nx; ix++)
		    {

		      double xpdflh=0;
		      double dexp=double(ix)/(nx-1);
		      x[ix] = xmin*pow(xmax/xmin,dexp);
		     		
		      if(l<3){
			int ifl=flavmap[l];
			xpdflh=LHAPDF::xfx(x[ix],Q,ifl);
		      }
		      if(l==3){ // Singlet
			xpdflh =0;
			for(unsigned i=1;i<7;i++){
			  xpdflh=xpdflh + LHAPDF::xfx(x[ix],Q,i) + LHAPDF::xfx(x[ix],Q,-i);
			}
		      }
		      if(l==4){ // Triplet
			xpdflh=LHAPDF::xfx(x[ix],Q,2) + LHAPDF::xfx(x[ix],Q,-2)
			  -  LHAPDF::xfx(x[ix],Q,1) - LHAPDF::xfx(x[ix],Q,-1) ;
		      }
		      if(l==5){ // Valence
			xpdflh =0;
			for(unsigned i=1;i<7;i++){
			  xpdflh=xpdflh + LHAPDF::xfx(x[ix],Q,i) - LHAPDF::xfx(x[ix],Q,-i);
			}
		      }
		      if(l==6){ // Triplet
			xpdflh=LHAPDF::xfx(x[ix],Q,3) + LHAPDF::xfx(x[ix],Q,-3);
		      }
		      if(l==7){ // Triplet
			xpdflh=LHAPDF::xfx(x[ix],Q,-2);
		      }
		      if(l==8){ // Triplet
			xpdflh=LHAPDF::xfx(x[ix],Q,-1);
		      }
		      if(l==9){ // Triplet
			xpdflh=LHAPDF::xfx(x[ix],Q,4)+LHAPDF::xfx(x[ix],Q,-4);
		      }
		        if(l==10){ 
			xpdflh=LHAPDF::xfx(x[ix],Q,-1)-LHAPDF::xfx(x[ix],Q,-2);
		      }
		        if(l==11){ 
			xpdflh=LHAPDF::xfx(x[ix],Q,3)-LHAPDF::xfx(x[ix],Q,-3);
		      }
			 if(l==12){ 
			 xpdflh=LHAPDF::xfx(x[ix],Q,3);
		       }
		       if(l==13){ 
			 xpdflh=LHAPDF::xfx(x[ix],Q,-3);
		       }

		      //		      cout<<nrep+1<<" "<<ix<<" "<<xpdflh<<endl;

		      xpdflh_ref[ix][nrep] = xpdflh;
		      
		    } //end loop over ix
		  
		} // end loop over replicas
	      
	      // Now loop again over x
	      for(int ix=0; ix<nx; ix++)
		{

		  cv[ix]=xpdflh_ref[ix][0];
		  err[ix]=0.0;
		  for (int nrep = 1; nrep <= repfinal; nrep++)
		    {
		      err[ix] += pow(xpdflh_ref[ix][nrep] - xpdflh_ref[ix][0],2.0);
		    }
		  
		  err[ix] = pow(err[ix],0.5);

		  if (iPdf == 0) CVpdf[ix] = cv[ix];
		  if( CVpdf[ix] < pdfmin ) CVpdf[ix]=pdfmin;
		  if(  cv[ix] < pdfmin )  cv[ix]=pdfmin;
		  
		  cvLux[iPdf]->SetPoint(ix, x[ix], cv[ix]/CVpdf[ix]);
		  cvLux[iPdf]->SetPointError(ix, 0.0, err[ix]/CVpdf[ix]);
		  cvLuxCT[iPdf]->SetPoint(ix, x[ix], cv[ix]/CVpdf[ix]);
		  cvLuxUp[iPdf]->SetPoint(ix, x[ix], (cv[ix]+err[ix])/CVpdf[ix]);
		  cvLuxDn[iPdf]->SetPoint(ix, x[ix], (cv[ix]-err[ix])/CVpdf[ix]);
		  cvLux_abs[iPdf]->SetPoint(ix, x[ix], cv[ix]);
		  cvLux_abs[iPdf]->SetPointError(ix, 0.0, err[ix]);
		  cvLuxCT_abs[iPdf]->SetPoint(ix, x[ix], cv[ix]);
		  cvLuxUp_abs[iPdf]->SetPoint(ix, x[ix], (cv[ix]+err[ix]));
		  cvLuxDn_abs[iPdf]->SetPoint(ix, x[ix], (cv[ix]-err[ix]));
		  
		}
            }

	
	   else if (pdftype[iPdf] == "ER_MSTW")
            {

	      LHAPDF::initPDFSet(pdfcvname[iPdf]);
	      int repfinal = LHAPDF::numberPDF();

	      double xpdflh_ref[nx][1001]={{0.0}};
	      
	      for (int nrep = 0; nrep <= repfinal; nrep++)
		{
		  LHAPDF::initPDF(nrep);
		  
		  for (int ix = 0; ix < nx; ix++)
		    {

		      double xpdflh=0;
		      double dexp=double(ix)/(nx-1);
		      x[ix] = xmin*pow(xmax/xmin,dexp);
		     		
		      if(l<3){
			int ifl=flavmap[l];
			xpdflh=LHAPDF::xfx(x[ix],Q,ifl);
		      }
		      if(l==3){ // Singlet
			xpdflh =0;
			for(unsigned i=1;i<7;i++){
			  xpdflh=xpdflh + LHAPDF::xfx(x[ix],Q,i) + LHAPDF::xfx(x[ix],Q,-i);
			}
		      }
		      if(l==4){ // Triplet
			xpdflh=LHAPDF::xfx(x[ix],Q,2) + LHAPDF::xfx(x[ix],Q,-2)
			  -  LHAPDF::xfx(x[ix],Q,1) - LHAPDF::xfx(x[ix],Q,-1) ;
		      }
		      if(l==5){ // Valence
			xpdflh =0;
			for(unsigned i=1;i<7;i++){
			  xpdflh=xpdflh + LHAPDF::xfx(x[ix],Q,i) - LHAPDF::xfx(x[ix],Q,-i);
			}
		      }
		      if(l==6){ // Triplet
			xpdflh=LHAPDF::xfx(x[ix],Q,3) + LHAPDF::xfx(x[ix],Q,-3);
		      }
		      if(l==7){ // Triplet
			xpdflh=LHAPDF::xfx(x[ix],Q,-2);
		      }
		      if(l==8){ // Triplet
			xpdflh=LHAPDF::xfx(x[ix],Q,-1);
		      }
		      if(l==9){ // Total charm
			xpdflh=LHAPDF::xfx(x[ix],Q,4)+LHAPDF::xfx(x[ix],Q,-4);
		      }
		        if(l==10){ 
			xpdflh=LHAPDF::xfx(x[ix],Q,-1)-LHAPDF::xfx(x[ix],Q,-2);
		      }
		        if(l==11){ 
			xpdflh=LHAPDF::xfx(x[ix],Q,3)-LHAPDF::xfx(x[ix],Q,-3);
		      }
			 if(l==12){ 
			 xpdflh=LHAPDF::xfx(x[ix],Q,3);
		       }
		       if(l==13){ 
			 xpdflh=LHAPDF::xfx(x[ix],Q,-3);
		       }

		      //		      cout<<nrep+1<<" "<<ix<<" "<<xpdflh<<endl;

		      xpdflh_ref[ix][nrep] = xpdflh;
		      
		    } //end loop over ix
		  
		} // end loop over replicas
	   

	   
	      
	      // Now loop again over x
	      for(int ix=0; ix<nx; ix++)
		{

		
		  cv[ix]=xpdflh_ref[ix][0];
		  err[ix]=0.0;
		  for (int nrep = 1; nrep <= repfinal/2; nrep++)
		    {
		     		      err[ix] += pow(xpdflh_ref[ix][2*nrep-1] - xpdflh_ref[ix][2*nrep],2.0);
		    }
		  err[ix] = 0.5*pow(err[ix],0.5);
		  
		  
		  if (iPdf == 0) CVpdf[ix] = cv[ix];
		  if( CVpdf[ix] < pdfmin ) CVpdf[ix]=pdfmin;
		  if(  cv[ix] < pdfmin )  cv[ix]=pdfmin;
		  
		  cvLux[iPdf]->SetPoint(ix, x[ix], cv[ix]/CVpdf[ix]);
		  cvLux[iPdf]->SetPointError(ix, 0.0, err[ix]/CVpdf[ix]);
		  cvLuxCT[iPdf]->SetPoint(ix, x[ix], cv[ix]/CVpdf[ix]);
		  cvLuxUp[iPdf]->SetPoint(ix, x[ix], (cv[ix]+err[ix])/CVpdf[ix]);
		  cvLuxDn[iPdf]->SetPoint(ix, x[ix], (cv[ix]-err[ix])/CVpdf[ix]);
		  cvLux_abs[iPdf]->SetPoint(ix, x[ix], cv[ix]);
		  cvLux_abs[iPdf]->SetPointError(ix, 0.0, err[ix]);
		  cvLuxCT_abs[iPdf]->SetPoint(ix, x[ix], cv[ix]);
		  cvLuxUp_abs[iPdf]->SetPoint(ix, x[ix], (cv[ix]+err[ix]));
		  cvLuxDn_abs[iPdf]->SetPoint(ix, x[ix], (cv[ix]-err[ix]));

		  // Relative error: PDF error normalized to the corresponding central value
		  cvLuxE[iPdf]->SetPoint(ix, x[ix], err[ix]/cv[ix]);
		  //cvLuxE[iPdf]->SetPoint(ix, x[ix], cv[ix]);
		  
		}
            }

	   else if (pdftype[iPdf] == "ER_CTEQ")
            {

	      LHAPDF::initPDFSet(pdfcvname[iPdf]);
	      int repfinal = LHAPDF::numberPDF();

	      double xpdflh_ref[nx][1001]={{0.0}};
	      
	      for (int nrep = 0; nrep <= repfinal; nrep++)
		{
		  LHAPDF::initPDF(nrep);
		  
		  for (int ix = 0; ix < nx; ix++)
		    {

		      double xpdflh=0;
		      double dexp=double(ix)/(nx-1);
		      x[ix] = xmin*pow(xmax/xmin,dexp);
		     		
		      if(l<3){
			int ifl=flavmap[l];
			xpdflh=LHAPDF::xfx(x[ix],Q,ifl);
		      }
		      if(l==3){ // Singlet
			xpdflh =0;
			for(unsigned i=1;i<7;i++){
			  xpdflh=xpdflh + LHAPDF::xfx(x[ix],Q,i) + LHAPDF::xfx(x[ix],Q,-i);
			}
		      }
		      if(l==4){ // Triplet
			xpdflh=LHAPDF::xfx(x[ix],Q,2) + LHAPDF::xfx(x[ix],Q,-2)
			  -  LHAPDF::xfx(x[ix],Q,1) - LHAPDF::xfx(x[ix],Q,-1) ;
		      }
		      if(l==5){ // Valence
			xpdflh =0;
			for(unsigned i=1;i<7;i++){
			  xpdflh=xpdflh + LHAPDF::xfx(x[ix],Q,i) - LHAPDF::xfx(x[ix],Q,-i);
			}
		      }
		      if(l==6){ // Triplet
			xpdflh=LHAPDF::xfx(x[ix],Q,3) + LHAPDF::xfx(x[ix],Q,-3);
		      }
		      if(l==7){ // Triplet
			xpdflh=LHAPDF::xfx(x[ix],Q,-2);
		      }
		      if(l==8){ // Triplet
			xpdflh=LHAPDF::xfx(x[ix],Q,-1);
		      }
		      if(l==9){ // Total charm
			xpdflh=LHAPDF::xfx(x[ix],Q,4)+LHAPDF::xfx(x[ix],Q,-4);
		      }
		        if(l==10){ 
			xpdflh=LHAPDF::xfx(x[ix],Q,-1)-LHAPDF::xfx(x[ix],Q,-2);
		      }
		        if(l==11){ 
			xpdflh=LHAPDF::xfx(x[ix],Q,3)-LHAPDF::xfx(x[ix],Q,-3);
		      }
			 if(l==12){ 
			 xpdflh=LHAPDF::xfx(x[ix],Q,3);
		       }
		       if(l==13){ 
			 xpdflh=LHAPDF::xfx(x[ix],Q,-3);
		       }

		      //		      cout<<nrep+1<<" "<<ix<<" "<<xpdflh<<endl;

		      xpdflh_ref[ix][nrep] = xpdflh;
		      
		    } //end loop over ix
		  
		} // end loop over replicas
	      
	      // Now loop again over x
	      for(int ix=0; ix<nx; ix++)
		{

		  cv[ix]=xpdflh_ref[ix][0];
		  err[ix]=0.0;
		  for (int nrep = 1; nrep <= repfinal/2; nrep++)
		    {
		      err[ix] += pow(xpdflh_ref[ix][2*nrep-1] - xpdflh_ref[ix][2*nrep],2.0);
		    }
		  
		  err[ix] = 0.5*pow(err[ix],0.5)/1.642;

		  if (iPdf == 0) CVpdf[ix] = cv[ix];
		  if( CVpdf[ix] < pdfmin ) CVpdf[ix]=pdfmin;
		  if(  cv[ix] < pdfmin )  cv[ix]=pdfmin;
		  
		  cvLux[iPdf]->SetPoint(ix, x[ix], cv[ix]/CVpdf[ix]);
		  cvLux[iPdf]->SetPointError(ix, 0.0, err[ix]/CVpdf[ix]);
		  cvLuxCT[iPdf]->SetPoint(ix, x[ix], cv[ix]/CVpdf[ix]);
		  cvLuxUp[iPdf]->SetPoint(ix, x[ix], (cv[ix]+err[ix])/CVpdf[ix]);
		  cvLuxDn[iPdf]->SetPoint(ix, x[ix], (cv[ix]-err[ix])/CVpdf[ix]);
		  cvLux_abs[iPdf]->SetPoint(ix, x[ix], cv[ix]);
		  cvLux_abs[iPdf]->SetPointError(ix, 0.0, err[ix]);
		  cvLuxCT_abs[iPdf]->SetPoint(ix, x[ix], cv[ix]);
		  cvLuxUp_abs[iPdf]->SetPoint(ix, x[ix], (cv[ix]+err[ix]));
		  cvLuxDn_abs[iPdf]->SetPoint(ix, x[ix], (cv[ix]-err[ix]));

		   // Relative error: PDF error normalized to the corresponding central value
		  cvLuxE[iPdf]->SetPoint(ix, x[ix], err[ix]/cv[ix]);
		  //cvLuxE[iPdf]->SetPoint(ix, x[ix], err[ix]);

		  
		}
            }
	  
	  else{
	    cout<<"Invalid option for PDF errors"<<endl;
	    exit(-10);
	  }
      

	  if (iPdf == 0)
	    {
	      
	      c.push_back(new TCanvas());
	      c[0]->SetTickx();
	      c[0]->SetTicky();
	      //c[0]->SetLogx();

	      cE.push_back(new TCanvas());
	      cE[0]->SetTickx();
	      cE[0]->SetTicky();
	      //cE[0]->SetLogx();
	      
	     
	      if(iabs==0){
		leg.push_back(new TLegend(0.11,0.66,0.45,0.89));
		leg[0]->SetLineStyle(1);
		leg[0]->SetBorderSize(1);
		leg[0]->SetFillColor(kWhite);

		legE.push_back(new TLegend(0.11,0.66,0.45,0.89));
		legE[0]->SetLineStyle(1);
		legE[0]->SetBorderSize(1);
		legE[0]->SetFillColor(kWhite);
	      }
	      if(iabs==1){
		leg_abs.push_back(new TLegend(0.11,0.66,0.45,0.89));
		leg_abs[0]->SetLineStyle(1);
		leg_abs[0]->SetBorderSize(1);
		leg_abs[0]->SetFillColor(kWhite);
	      }

	    }
	  

	  if(iabs==0) { 
	    cvLux[iPdf]->SetTitle(title.c_str());
	    cvLux[iPdf]->SetLineWidth(2);
	    cvLux[iPdf]->SetLineColor(color2[iPdf]);
	    cvLux[iPdf]->SetLineStyle(2);
	    cvLux[iPdf]->SetFillColor(color[iPdf]);
	    cvLux[iPdf]->SetFillStyle(histoFillStyle[iPdf]);
	    
	    cvLux[iPdf]->GetXaxis()->SetTitle("       x  ");
	    cvLux[iPdf]->GetXaxis()->SetTitleSize(0.05);
	    cvLux[iPdf]->GetXaxis()->SetTitleOffset(0.8);
	    cvLux[iPdf]->GetXaxis()->SetLabelSize(0.042);
	    cvLux[iPdf]->GetYaxis()->SetLabelSize(0.042);
	    cvLux[iPdf]->GetXaxis()->SetLimits(xmin, xmax);
	    cvLux[iPdf]->GetXaxis()->CenterTitle(true);
	    cvLux[iPdf]->GetYaxis()->SetTitle(titleY[l].c_str());
	    cvLux[iPdf]->GetYaxis()->CenterTitle(true);
	    cvLux[iPdf]->GetYaxis()->SetRangeUser(0.86,1.17);
	    if(l==6)cvLux[iPdf]->GetYaxis()->SetRangeUser(0.80,1.30);
	    //if(l==9)cvLux[iPdf]->GetYaxis()->SetRangeUser(0.50,2.00);
	    if(l==9)cvLux[iPdf]->GetYaxis()->SetRangeUser(0.75,1.30);
	    if(l==11)cvLux[iPdf]->GetYaxis()->SetRangeUser(0,2);
	    if(l==12||l==13)cvLux[iPdf]->GetYaxis()->SetRangeUser(0.70,1.27);
	    	    
	    cvLux[iPdf]->GetYaxis()->SetTitleSize(0.05);
	    
	    cvLuxCT[iPdf]->SetLineColor(color2[iPdf]);
	    cvLuxCT[iPdf]->SetLineWidth(2);
	    cvLuxCT[iPdf]->SetLineStyle(2);
	    cvLuxUp[iPdf]->SetLineColor(color2[iPdf]);
	    cvLuxUp[iPdf]->SetLineWidth(2);
	    cvLuxUp[iPdf]->SetLineStyle(1);
	    cvLuxDn[iPdf]->SetLineColor(color2[iPdf]);
	    cvLuxDn[iPdf]->SetLineWidth(2);
	    cvLuxDn[iPdf]->SetLineStyle(1);



	    cvLuxE[iPdf]->SetTitle(title.c_str());
	    cvLuxE[iPdf]->SetLineWidth(3);
	    cvLuxE[iPdf]->SetLineColor(color2[iPdf]);
	    cvLuxE[iPdf]->SetLineStyle(iPdf+1);
	    cvLuxE[iPdf]->GetXaxis()->SetTitle("       x  ");
	    cvLuxE[iPdf]->GetXaxis()->SetTitleSize(0.040);
	    cvLuxE[iPdf]->GetXaxis()->SetTitleOffset(0.8);
	    cvLuxE[iPdf]->GetXaxis()->SetLabelSize(0.037);
	    cvLuxE[iPdf]->GetYaxis()->SetLabelSize(0.040);
	    cvLuxE[iPdf]->GetXaxis()->SetLimits(xmin, xmax);
	    cvLuxE[iPdf]->GetXaxis()->CenterTitle(true);
	    cvLuxE[iPdf]->GetYaxis()->SetTitle(titleYE[l].c_str());
	    cvLuxE[iPdf]->GetYaxis()->CenterTitle(true);
	    cvLuxE[iPdf]->GetYaxis()->SetRangeUser(0.00,0.12);
	    //cvLuxE[iPdf]->GetYaxis()->SetRangeUser(0.50,2.0);
	    if(l==6 || l==12 || l==13) cvLuxE[iPdf]->GetYaxis()->SetRangeUser(0.00,0.16);
	    cvLuxE[iPdf]->GetYaxis()->SetTitleSize(0.05);
	    cvLuxE[iPdf]->GetXaxis()->SetTitleOffset(0.8);
	    
	    if (iPdf == 0 )
	      {
		c[0]->cd();
		cvLux[iPdf]->Draw("a3");
		cvLuxCT[iPdf]->Draw("L");
		cvLuxUp[iPdf]->Draw("L");
		cvLuxDn[iPdf]->Draw("L");
		
		leg[0]->AddEntry(cvLux[iPdf], TString(legendname[iPdf]),"fl");


		cE[0]->cd();
		cvLuxE[iPdf]->Draw("AL");
		legE[0]->AddEntry(cvLuxE[iPdf], TString(legendname[iPdf]),"L");
		
	      }
	    else 
	      {
		c[0]->cd();
		cvLux[iPdf]->Draw("3,same");
		cvLuxCT[iPdf]->Draw("L,same");
		cvLuxUp[iPdf]->Draw("L,same");
		cvLuxDn[iPdf]->Draw("L,same");
		
		leg[0]->AddEntry(cvLux[iPdf], TString(legendname[iPdf]),"fl");


		cE[0]->cd();
		cvLuxE[iPdf]->Draw("L,same");
		legE[0]->AddEntry(cvLux[iPdf], TString(legendname[iPdf]),"L");
	      }

	  }
	
	else if(iabs==1){

	  cvLux_abs[iPdf]->SetTitle(title.c_str());
	   
	  cvLux_abs[iPdf]->SetLineWidth(2);
	  cvLux_abs[iPdf]->SetLineColor(color2[iPdf]);
	  cvLux_abs[iPdf]->SetLineStyle(2);
	  cvLux_abs[iPdf]->SetFillColor(color[iPdf]);
	  cvLux_abs[iPdf]->SetFillStyle(histoFillStyle[iPdf]);
	  
	  cvLux_abs[iPdf]->GetXaxis()->SetTitle("       x  ");
	  cvLux_abs[iPdf]->GetXaxis()->SetTitleSize(0.05);
	  cvLux_abs[iPdf]->GetXaxis()->SetTitleOffset(0.8);
	  cvLux_abs[iPdf]->GetXaxis()->SetLabelSize(0.036);
	  cvLux_abs[iPdf]->GetXaxis()->SetLimits(xmin, xmax);
	  cvLux_abs[iPdf]->GetXaxis()->CenterTitle(true);
	  cvLux_abs[iPdf]->GetYaxis()->SetTitle(titleY_abs[l].c_str());
	  cvLux_abs[iPdf]->GetYaxis()->CenterTitle(true);
	  cvLux_abs[iPdf]->GetYaxis()->SetRangeUser(-1,5);
	  if(l==0)cvLux_abs[iPdf]->GetYaxis()->SetRangeUser(-1,6);
	 
	   // up quark
	  if(l==1)cvLux_abs[iPdf]->GetYaxis()->SetRangeUser(-0.003,0.023);
	  if(l==1)cvLux_abs[iPdf]->GetXaxis()->SetLimits(0.4, xmax);

	  // down quark
	  if(l==2)cvLux_abs[iPdf]->GetYaxis()->SetRangeUser(-0.003,0.023);
	  if(l==2)cvLux_abs[iPdf]->GetXaxis()->SetLimits(0.4, xmax);


	  if(l==3)cvLux_abs[iPdf]->GetYaxis()->SetRangeUser(0,4.5);

	  // Quark triplet
	  if(l==4)cvLux_abs[iPdf]->GetYaxis()->SetRangeUser(-0.05,0.50);
	  if(l==4)cvLux_abs[iPdf]->GetXaxis()->SetLimits(5e-4, xmax);
	  
	  if(l==5)cvLux_abs[iPdf]->GetYaxis()->SetRangeUser(-0.1,1.6);
	  if(l==6)cvLux_abs[iPdf]->GetYaxis()->SetRangeUser(-0.1,1.0);
	  if(l==6)cvLux_abs[iPdf]->GetXaxis()->SetLimits(1e-5, xmax);

	  
	  
	  if(l==9)cvLux_abs[iPdf]->GetYaxis()->SetRangeUser(-0.015,0.06);

	  // DeltaS
	  if(l==10)cvLux_abs[iPdf]->GetYaxis()->SetRangeUser(-0.025,0.06);
	  if(l==10)cvLux_abs[iPdf]->GetXaxis()->SetLimits(5e-4, xmax);
	  
	  cvLux_abs[iPdf]->GetYaxis()->SetTitleSize(0.05);
	  cvLux_abs[iPdf]->GetYaxis()->SetLabelSize(0.036);
	  
	  cvLuxCT_abs[iPdf]->SetLineColor(color2[iPdf]);
	  cvLuxCT_abs[iPdf]->SetLineWidth(2);
	  cvLuxCT_abs[iPdf]->SetLineStyle(2);
	  cvLuxUp_abs[iPdf]->SetLineColor(color2[iPdf]);
	  cvLuxUp_abs[iPdf]->SetLineWidth(2);
	  cvLuxUp_abs[iPdf]->SetLineStyle(1);
	    cvLuxDn_abs[iPdf]->SetLineColor(color2[iPdf]);
	    cvLuxDn_abs[iPdf]->SetLineWidth(2);
	    cvLuxDn_abs[iPdf]->SetLineStyle(1);
	    
	    if (iPdf == 0 )
	      {
		c[0]->cd();
		cvLux_abs[iPdf]->Draw("a3");
		cvLuxCT_abs[iPdf]->Draw("c");
		cvLuxUp_abs[iPdf]->Draw("c");
		cvLuxDn_abs[iPdf]->Draw("c");
		
		leg_abs[0]->AddEntry(cvLux_abs[iPdf], TString(legendname[iPdf]),"fl");
	      }
	    else 
	      {
		c[0]->cd();
		cvLux_abs[iPdf]->Draw("3,same");
		cvLuxCT_abs[iPdf]->Draw("c,same");
		cvLuxUp_abs[iPdf]->Draw("c,same");
		cvLuxDn_abs[iPdf]->Draw("c,same");
		
		leg_abs[0]->AddEntry(cvLux_abs[iPdf], TString(legendname[iPdf]),"fl");
	      }
	    
	  }
	  
	    
	} // end loop over iPDF


	
	  if(iabs==0){
	    c[0]->cd();
	    leg[0]->Draw();
	    	    c[0]->SaveAs(outfilename[l].c_str());
	    c[0]->SaveAs(outfilename2[l].c_str());


	    cE[0]->cd();
	    legE[0]->Draw();
	      cE[0]->SaveAs(outfilenameE[l].c_str());
	    cE[0]->SaveAs(outfilename2E[l].c_str());
	  }
	  else{
	    c[0]->cd();
	    leg_abs[0]->Draw();
	    c[0]->SaveAs(outfilename_abs[l].c_str());
	    c[0]->SaveAs(outfilename2_abs[l].c_str());
	  }
	  
      } // end loop over iabs
    } // end loop over lumi

  }

  return 0;
}


