
#include "lumiintegral.h"

int main(int argc, char **argv)
{  
  cout << "\n ****************************************\n";
  cout <<   " * Computation of partonic luminosities *\n";
  cout <<   " ****************************************\n\n";
  cout << endl;

  //////////////////////////
  // Important variables  //
  //////////////////////////
  double eps = 5e-2;        //! Relative error integration
  double S = pow(13e3, 2.0); //! LHC center of mass energy
  double mHmin = 800;        //! higgs mass of 120 GeV
  double mHmax = 5e3;       //! Higgs mass of 300 GeV
  int NmH = 40;             //! Number of points to calculate

  double const lumi_min=1e-14;

  for(int iplot=0;iplot<1;iplot++){
    
    // Init PDF for CV and Error
    vector<string> pdfcvname;
    vector<string> pdferrname;
    vector<string> legendname;
    vector<string> pdftype;
    
    vector<string> outfilename;
    vector<string> outfilename2;
    vector<string> outfilenameE;
    vector<string> outfilename2E;
    string filename;
    string title;
    string titleE;

    //******************************************************
    // NNPDF3.1 NNLO vs global fits
    //******************************************************
    if(iplot==0){

      pdfcvname.push_back("MMHT2014nnlo68cl");
      pdferrname.push_back("MMHT2014nnlo68cl");
      legendname.push_back("MMHT14");

      pdfcvname.push_back("ABMP16als118_5_nnlo");
      pdferrname.push_back("ABMP16als118_5_nnlo");
      legendname.push_back("ABMP16");
      
      pdfcvname.push_back("CT14nnlo");
      pdferrname.push_back("CT14nnlo");
      legendname.push_back("CT14");

      pdfcvname.push_back("NNPDF31_nnlo_as_0118");
      pdferrname.push_back("NNPDF31_nnlo_as_0118");
      legendname.push_back("NNPDF3.1");
            
      filename="lumi-tev";
      
      title="LHC 13 TeV, NNLO, #alpha_{s}=0.118";

      pdftype.push_back("ER_MSTW");
      pdftype.push_back("ER_HESSIAN");
      pdftype.push_back("ER_CTEQ");
      pdftype.push_back("ER_MC");
            
    }

         

 
  string f1="gg_"+filename+".eps";
  outfilename.push_back(f1.c_str());
  f1="qq_"+filename+".eps";
  outfilename.push_back(f1.c_str());
  f1="q2_"+filename+".eps";
  outfilename.push_back(f1.c_str());
  f1="qg_"+filename+".eps";
  outfilename.push_back(f1.c_str());

  string f2="gg_"+filename+".pdf";
  outfilename2.push_back(f2.c_str());
  f2="qq_"+filename+".pdf";
  outfilename2.push_back(f2.c_str());
  f2="q2_"+filename+".pdf";
  outfilename2.push_back(f2.c_str());
  f2="qg_"+filename+".pdf";
  outfilename2.push_back(f2.c_str());


  f1="ggERR_"+filename+".eps";
  outfilenameE.push_back(f1.c_str());
  f1="qqERR_"+filename+".eps";
  outfilenameE.push_back(f1.c_str());
  f1="q2ERR_"+filename+".eps";
  outfilenameE.push_back(f1.c_str());
  f1="qgERR_"+filename+".eps";
  outfilenameE.push_back(f1.c_str());

  f2="ggERR_"+filename+".pdf";
  outfilename2E.push_back(f2.c_str());
  f2="qqERR_"+filename+".pdf";
  outfilename2E.push_back(f2.c_str());
  f2="q2ERR_"+filename+".pdf";
  outfilename2E.push_back(f2.c_str());
  f2="qgERR_"+filename+".pdf";
  outfilename2E.push_back(f2.c_str());

  
  vector<Color_t> color;
  color.push_back(kGreen);
  color.push_back(kRed);
  color.push_back(kBlue);
  color.push_back(1);
  
  vector<Color_t> color2;
  color2.push_back(kGreen+2);
  color2.push_back(kRed+1);
  color2.push_back(kBlue+2);
  color2.push_back(1);
  
   
  vector<string> lumis;
  lumis.push_back("GG");
  lumis.push_back("QQ");
  lumis.push_back("Q2");
  lumis.push_back("QG");
     
  vector<string> titleY;
  titleY.push_back("Gluon - Gluon Luminosity");
  titleY.push_back("Quark - Antiquark Luminosity");
  titleY.push_back("Quark - Quark Luminosity");
  titleY.push_back("Quark - Gluon Luminosity");

  vector<string> titleYE;
  titleYE.push_back("Gluon - Gluon Luminosity (RelErr)");
  titleYE.push_back("Quark - Antiquark Luminosity (RelErr)");
  titleYE.push_back("Quark - Quark Luminosity (RelErr)");
  titleYE.push_back("Quark - Gluon Luminosity (RelErr)");
 
  cout << "\n Computing parton luminosities" << endl;
  cout << endl;

  // Max number of replicas
  int const nrepmax=250;

  // Set low level of verbosity
  LHAPDF::setVerbosity(0);
  
  LumiIntegral *lum = new LumiIntegral(eps);
  const int histoFillStyle[] = { 1001, 3003 , 3004, 3005};
  
  for (size_t l = 0; l < lumis.size(); l++){
    //for (size_t l = 1; l < 2; l++){
    
    vector<TCanvas*> c;
    vector<TLegend*> leg;
    vector<TGraphErrors*> cvLux;
    vector<TGraphErrors*> cvLuxUp;
    vector<TGraphErrors*> cvLuxDn;
    vector<TGraphErrors*> cvLuxCT;


    vector<TCanvas*> cE;
    vector<TLegend*> legE;
    vector<TGraph*> cvLuxE;

    
    double *CVpdf = new double[NmH];
    for (int w = 0; w < NmH; w++)
      CVpdf[w] = 1.0;
    
    for (size_t iPdf = 0; iPdf < pdfcvname.size(); iPdf++){
      
      double *xmH = new double[NmH];
      
      cvLux.push_back(new TGraphErrors(NmH));
      cvLuxUp.push_back(new TGraphErrors(NmH));
      cvLuxDn.push_back(new TGraphErrors(NmH));
      cvLuxCT.push_back(new TGraphErrors(NmH));

      cvLuxE.push_back(new TGraph(NmH));
      
      LHAPDF::initPDFSet(pdfcvname[iPdf]);
      int repfinal = LHAPDF::numberPDF();
      // Begin loop over replicas
      double lumi[NmH][nrepmax];
      for (int irep = 0; irep <= repfinal; irep++){

	std::cout<<"l, iPDF, irep "<<l<<" "<<iPdf<<" "<<irep<<endl;
	
	// Initialize replica
	LHAPDF::initPDF(irep);
	
	// Loop over Higgs masses
	for (int imH =0; imH < NmH; imH++){
	  double mH = mHmin * pow(mHmax/mHmin, double(imH)/(NmH-1));
	  
	  lumi[imH][irep] = lum->getLum(mH, S, lumis[l]);

	  // Be careful to avoid negative luminosities
	  if(lumi[imH][irep] < lumi_min) lumi[imH][irep]=lumi_min; 
	  
	  xmH[imH]   = mH;//sqrt(mH*mH/S);
	} 
	
      } // end loop over replicas

      // Loop again over Higgs mass
      for (int imH = 0; imH < NmH; imH++){

	double cv=0;
	double sigma=0;
	if(pdftype[iPdf] == "ER_MC"){

	  for (int irep = 1; irep <= repfinal; irep++){
	    
	    cv += lumi[imH][irep] / repfinal;
	    sigma += pow( lumi[imH][irep], 2.0) / repfinal;
	    
	  }
	  sigma = pow(sigma - cv*cv, 0.5);
	}
	else if(pdftype[iPdf] == "ER_CTEQ" || pdftype[iPdf] == "ER_MSTW" ){

	  cv=lumi[imH][0];
	  
	  for (int irep = 1; irep <= repfinal/2; irep++){
	    
	    sigma += pow( lumi[imH][2*irep-1]-lumi[imH][2*irep], 2.0);
	    
	  }
	  sigma = 0.5*pow(sigma, 0.5);
	  if(pdftype[iPdf] == "ER_CTEQ") sigma *= (1/1.642);
	}
	else if(pdftype[iPdf] == "ER_HESSIAN" ){

	  cv=lumi[imH][0];
	  
	  for (int irep = 1; irep <= repfinal; irep++){
	    
	    sigma += pow( lumi[imH][irep]-lumi[imH][0], 2.0);
	    
	  }
	  sigma = pow(sigma, 0.5);
	}
	  
	// Here one can choose the normalizations
	if (iPdf == 0) CVpdf[imH] = cv;
	
	cvLux[iPdf]->SetPoint(imH, xmH[imH]/1e3, cv/CVpdf[imH]);
	cvLux[iPdf]->SetPointError(imH, 0.0, sigma/CVpdf[imH]);
	cvLuxCT[iPdf]->SetPoint(imH, xmH[imH]/1e3, cv/CVpdf[imH]);
	cvLuxUp[iPdf]->SetPoint(imH, xmH[imH]/1e3, (cv+sigma)/CVpdf[imH]);
	cvLuxDn[iPdf]->SetPoint(imH, xmH[imH]/1e3, (cv-sigma)/CVpdf[imH]);

	cvLuxE[iPdf]->SetPoint(imH, xmH[imH], sigma/CVpdf[imH]);
	cout<<xmH[imH]<<" "<< sigma/CVpdf[imH]<<endl;
	
      } // End loop over the Higgs masses

      if (iPdf == 0){
	
	c.push_back(new TCanvas());
	c[0]->SetTickx();
	c[0]->SetTicky();
	//c[0]->SetLogx();
	//c[0]->SetLogy();
	
	leg.push_back(new TLegend(0.11,0.60,0.43,0.89));
	leg[0]->SetLineStyle(1);
	leg[0]->SetBorderSize(1);
	leg[0]->SetFillColor(kWhite);


	cE.push_back(new TCanvas());
	cE[0]->SetTickx();
	cE[0]->SetTicky();
	cE[0]->SetLogx();
	//c[0]->SetLogy();
	
	legE.push_back(new TLegend(0.11,0.65,0.43,0.89));
	legE[0]->SetLineStyle(1);
	legE[0]->SetBorderSize(1);
	legE[0]->SetFillColor(kWhite);

      }
	
      cvLux[iPdf]->SetTitle(title.c_str());
      cvLux[iPdf]->SetLineWidth(2);
      cvLux[iPdf]->SetLineColor(color2[iPdf]);
      cvLux[iPdf]->SetLineStyle(2);
      cvLux[iPdf]->SetFillColor(color[iPdf]);
      cvLux[iPdf]->SetFillStyle(histoFillStyle[iPdf]);
      
      cvLux[iPdf]->GetXaxis()->SetTitle("M_{X} ( GeV )");
      cvLux[iPdf]->GetXaxis()->SetTitleSize(0.05);
      cvLux[iPdf]->GetXaxis()->SetTitleOffset(0.8);
      cvLux[iPdf]->GetXaxis()->SetLabelSize(0.04);
      cvLux[iPdf]->GetXaxis()->SetLimits(mHmin/1e3, mHmax/1e3);
      cvLux[iPdf]->GetXaxis()->CenterTitle(true);
      cvLux[iPdf]->GetYaxis()->SetTitle(titleY[l].c_str());
      cvLux[iPdf]->GetYaxis()->CenterTitle(true);
      cvLux[iPdf]->GetYaxis()->SetRangeUser(0.55,1.39);
      cvLux[iPdf]->GetYaxis()->SetTitleSize(0.05);
      cvLux[iPdf]->GetYaxis()->SetLabelSize(0.05);
      
      cvLuxCT[iPdf]->SetLineColor(color2[iPdf]);
      cvLuxCT[iPdf]->SetLineWidth(2);
      cvLuxCT[iPdf]->SetLineStyle(2);
      cvLuxUp[iPdf]->SetLineColor(color2[iPdf]);
      cvLuxUp[iPdf]->SetLineWidth(2);
      cvLuxUp[iPdf]->SetLineStyle(1);
      cvLuxDn[iPdf]->SetLineColor(color2[iPdf]);
      cvLuxDn[iPdf]->SetLineWidth(2);
      cvLuxDn[iPdf]->SetLineStyle(1);


      cvLuxE[iPdf]->SetTitle(title.c_str());
      cvLuxE[iPdf]->SetLineWidth(2);
      cvLuxE[iPdf]->SetLineColor(color2[iPdf]);
      cvLuxE[iPdf]->SetLineStyle(iPdf+1);
                
      cvLuxE[iPdf]->GetXaxis()->SetTitle("M_{X} ( TeV )");
      cvLuxE[iPdf]->GetXaxis()->SetTitleSize(0.05);
      cvLuxE[iPdf]->GetXaxis()->SetTitleOffset(0.8);
      cvLuxE[iPdf]->GetXaxis()->SetLabelSize(0.05);
      cvLuxE[iPdf]->GetXaxis()->SetLimits(mHmin, mHmax);
      cvLuxE[iPdf]->GetXaxis()->CenterTitle(true);
      cvLuxE[iPdf]->GetYaxis()->SetTitle(titleYE[l].c_str());
      cvLuxE[iPdf]->GetYaxis()->CenterTitle(true);
      cvLuxE[iPdf]->GetYaxis()->SetRangeUser(0.0,0.125);
      cvLuxE[iPdf]->GetYaxis()->SetTitleSize(0.05);
      cvLuxE[iPdf]->GetYaxis()->SetLabelSize(0.05);
           
      if (iPdf == 0 ){
	c[0]->cd();
	cvLux[iPdf]->Draw("a3");
	cvLuxCT[iPdf]->Draw("l");
	cvLuxUp[iPdf]->Draw("l");
	cvLuxDn[iPdf]->Draw("l");
	leg[0]->AddEntry(cvLux[iPdf], TString(legendname[iPdf]),"fl");


	cE[0]->cd();
	cvLuxE[iPdf]->Draw("AL");
	legE[0]->AddEntry(cvLuxE[iPdf], TString(legendname[iPdf]),"L");
	
      }
      else {
	c[0]->cd();
	cvLux[iPdf]->Draw("3,same");
	cvLuxCT[iPdf]->Draw("l,same");
	cvLuxUp[iPdf]->Draw("l,same");
	cvLuxDn[iPdf]->Draw("l,same");
	leg[0]->AddEntry(cvLux[iPdf], TString(legendname[iPdf]),"fl");


	cE[0]->cd();
	cvLuxE[iPdf]->Draw("l,same");
	legE[0]->AddEntry(cvLuxE[iPdf], TString(legendname[iPdf]),"L");
	
      }
      
    } // End loop over PDF sets

    c[0]->cd();
    leg[0]->Draw();
    
    c[0]->SaveAs(outfilename[l].c_str());
    c[0]->SaveAs(outfilename2[l].c_str());


    cE[0]->cd();
    legE[0]->Draw();
    
    cE[0]->SaveAs(outfilenameE[l].c_str());
    cE[0]->SaveAs(outfilename2E[l].c_str());
    
  } // End loop over luminosities

  }
	
  return 0;
}

